"""


Audio System (aud)
******************

Audaspace (pronounced "outer space") is a high level audio library.


Basic Sound Playback
====================

This script shows how to use the classes: :class:`Device`, :class:`Factory` and
:class:`Handle`.

.. code::

  import aud

  device = aud.device()
  # load sound file (it can be a video file with audio)
  factory = aud.Factory('music.ogg')

  # play the audio, this return a handle to control play/pause
  handle = device.play(factory)
  # if the audio is not too big and will be used often you can buffer it
  factory_buffered = aud.Factory.buffer(factory)
  handle_buffered = device.play(factory_buffered)

  # stop the sounds (otherwise they play until their ends)
  handle.stop()
  handle_buffered.stop()

:data:`AP_LOCATION`

:data:`AP_ORIENTATION`

:data:`AP_PANNING`

:data:`AP_PITCH`

:data:`AP_VOLUME`

:data:`CHANNELS_INVALID`

:data:`CHANNELS_MONO`

:data:`CHANNELS_STEREO`

:data:`CHANNELS_STEREO_LFE`

:data:`CHANNELS_SURROUND4`

:data:`CHANNELS_SURROUND5`

:data:`CHANNELS_SURROUND51`

:data:`CHANNELS_SURROUND61`

:data:`CHANNELS_SURROUND71`

:data:`CODEC_AAC`

:data:`CODEC_AC3`

:data:`CODEC_FLAC`

:data:`CODEC_INVALID`

:data:`CODEC_MP2`

:data:`CODEC_MP3`

:data:`CODEC_OPUS`

:data:`CODEC_PCM`

:data:`CODEC_VORBIS`

:data:`CONTAINER_AC3`

:data:`CONTAINER_FLAC`

:data:`CONTAINER_INVALID`

:data:`CONTAINER_MATROSKA`

:data:`CONTAINER_MP2`

:data:`CONTAINER_MP3`

:data:`CONTAINER_OGG`

:data:`CONTAINER_WAV`

:data:`DISTANCE_MODEL_EXPONENT`

:data:`DISTANCE_MODEL_EXPONENT_CLAMPED`

:data:`DISTANCE_MODEL_INVALID`

:data:`DISTANCE_MODEL_INVERSE`

:data:`DISTANCE_MODEL_INVERSE_CLAMPED`

:data:`DISTANCE_MODEL_LINEAR`

:data:`DISTANCE_MODEL_LINEAR_CLAMPED`

:data:`FORMAT_FLOAT32`

:data:`FORMAT_FLOAT64`

:data:`FORMAT_INVALID`

:data:`FORMAT_S16`

:data:`FORMAT_S24`

:data:`FORMAT_S32`

:data:`FORMAT_U8`

:data:`RATE_11025`

:data:`RATE_16000`

:data:`RATE_192000`

:data:`RATE_22050`

:data:`RATE_32000`

:data:`RATE_44100`

:data:`RATE_48000`

:data:`RATE_8000`

:data:`RATE_88200`

:data:`RATE_96000`

:data:`RATE_INVALID`

:data:`STATUS_INVALID`

:data:`STATUS_PAUSED`

:data:`STATUS_PLAYING`

:data:`STATUS_STOPPED`

:class:`Device`

:class:`DynamicMusic`

:class:`Handle`

:class:`PlaybackManager`

:class:`Sequence`

:class:`SequenceEntry`

:class:`Sound`

:class:`Source`

:class:`ThreadPool`

:class:`error`

"""

import typing

AP_LOCATION: typing.Any = ...

"""

constant value 3

"""

AP_ORIENTATION: typing.Any = ...

"""

constant value 4

"""

AP_PANNING: typing.Any = ...

"""

constant value 1

"""

AP_PITCH: typing.Any = ...

"""

constant value 2

"""

AP_VOLUME: typing.Any = ...

"""

constant value 0

"""

CHANNELS_INVALID: typing.Any = ...

"""

constant value 0

"""

CHANNELS_MONO: typing.Any = ...

"""

constant value 1

"""

CHANNELS_STEREO: typing.Any = ...

"""

constant value 2

"""

CHANNELS_STEREO_LFE: typing.Any = ...

"""

constant value 3

"""

CHANNELS_SURROUND4: typing.Any = ...

"""

constant value 4

"""

CHANNELS_SURROUND5: typing.Any = ...

"""

constant value 5

"""

CHANNELS_SURROUND51: typing.Any = ...

"""

constant value 6

"""

CHANNELS_SURROUND61: typing.Any = ...

"""

constant value 7

"""

CHANNELS_SURROUND71: typing.Any = ...

"""

constant value 8

"""

CODEC_AAC: typing.Any = ...

"""

constant value 1

"""

CODEC_AC3: typing.Any = ...

"""

constant value 2

"""

CODEC_FLAC: typing.Any = ...

"""

constant value 3

"""

CODEC_INVALID: typing.Any = ...

"""

constant value 0

"""

CODEC_MP2: typing.Any = ...

"""

constant value 4

"""

CODEC_MP3: typing.Any = ...

"""

constant value 5

"""

CODEC_OPUS: typing.Any = ...

"""

constant value 8

"""

CODEC_PCM: typing.Any = ...

"""

constant value 6

"""

CODEC_VORBIS: typing.Any = ...

"""

constant value 7

"""

CONTAINER_AC3: typing.Any = ...

"""

constant value 1

"""

CONTAINER_FLAC: typing.Any = ...

"""

constant value 2

"""

CONTAINER_INVALID: typing.Any = ...

"""

constant value 0

"""

CONTAINER_MATROSKA: typing.Any = ...

"""

constant value 3

"""

CONTAINER_MP2: typing.Any = ...

"""

constant value 4

"""

CONTAINER_MP3: typing.Any = ...

"""

constant value 5

"""

CONTAINER_OGG: typing.Any = ...

"""

constant value 6

"""

CONTAINER_WAV: typing.Any = ...

"""

constant value 7

"""

DISTANCE_MODEL_EXPONENT: typing.Any = ...

"""

constant value 5

"""

DISTANCE_MODEL_EXPONENT_CLAMPED: typing.Any = ...

"""

constant value 6

"""

DISTANCE_MODEL_INVALID: typing.Any = ...

"""

constant value 0

"""

DISTANCE_MODEL_INVERSE: typing.Any = ...

"""

constant value 1

"""

DISTANCE_MODEL_INVERSE_CLAMPED: typing.Any = ...

"""

constant value 2

"""

DISTANCE_MODEL_LINEAR: typing.Any = ...

"""

constant value 3

"""

DISTANCE_MODEL_LINEAR_CLAMPED: typing.Any = ...

"""

constant value 4

"""

FORMAT_FLOAT32: typing.Any = ...

"""

constant value 36

"""

FORMAT_FLOAT64: typing.Any = ...

"""

constant value 40

"""

FORMAT_INVALID: typing.Any = ...

"""

constant value 0

"""

FORMAT_S16: typing.Any = ...

"""

constant value 18

"""

FORMAT_S24: typing.Any = ...

"""

constant value 19

"""

FORMAT_S32: typing.Any = ...

"""

constant value 20

"""

FORMAT_U8: typing.Any = ...

"""

constant value 1

"""

RATE_11025: typing.Any = ...

"""

constant value 11025

"""

RATE_16000: typing.Any = ...

"""

constant value 16000

"""

RATE_192000: typing.Any = ...

"""

constant value 192000

"""

RATE_22050: typing.Any = ...

"""

constant value 22050

"""

RATE_32000: typing.Any = ...

"""

constant value 32000

"""

RATE_44100: typing.Any = ...

"""

constant value 44100

"""

RATE_48000: typing.Any = ...

"""

constant value 48000

"""

RATE_8000: typing.Any = ...

"""

constant value 8000

"""

RATE_88200: typing.Any = ...

"""

constant value 88200

"""

RATE_96000: typing.Any = ...

"""

constant value 96000

"""

RATE_INVALID: typing.Any = ...

"""

constant value 0

"""

STATUS_INVALID: typing.Any = ...

"""

constant value 0

"""

STATUS_PAUSED: typing.Any = ...

"""

constant value 2

"""

STATUS_PLAYING: typing.Any = ...

"""

constant value 1

"""

STATUS_STOPPED: typing.Any = ...

"""

constant value 3

"""

class Device:

  """

  Device objects represent an audio output backend like OpenAL or SDL, but might also represent a file output or RAM buffer output.

  lock()

  Locks the device so that it's guaranteed, that no samples are read from the streams until :meth:`unlock` is called.
This is useful if you want to do start/stop/pause/resume some sounds at the same time.

  Note: The device has to be unlocked as often as locked to be able to continue playback.

  Warning: Make sure the time between locking and unlocking is as short as possible to avoid clicks.

  play(sound, keep=False)

  Plays a sound.

  stopAll()

  Stops all playing and paused sounds.

  unlock()

  Unlocks the device after a lock call, see :meth:`lock` for details.

  """

  channels: typing.Any = ...

  """

  The channel count of the device.

  """

  distance_model: typing.Any = ...

  """

  The distance model of the device.

  http://connect.creativelabs.com/openal/Documentation/OpenAL%201.1%20Specification.htm#_Toc199835864

  """

  doppler_factor: typing.Any = ...

  """

  The doppler factor of the device.
This factor is a scaling factor for the velocity vectors in doppler calculation. So a value bigger than 1 will exaggerate the effect as it raises the velocity.

  """

  format: typing.Any = ...

  """

  The native sample format of the device.

  """

  listener_location: typing.Any = ...

  """

  The listeners's location in 3D space, a 3D tuple of floats.

  """

  listener_orientation: typing.Any = ...

  """

  The listener's orientation in 3D space as quaternion, a 4 float tuple.

  """

  listener_velocity: typing.Any = ...

  """

  The listener's velocity in 3D space, a 3D tuple of floats.

  """

  rate: typing.Any = ...

  """

  The sampling rate of the device in Hz.

  """

  speed_of_sound: typing.Any = ...

  """

  The speed of sound of the device.
The speed of sound in air is typically 343.3 m/s.

  """

  volume: typing.Any = ...

  """

  The overall volume of the device.

  """

class DynamicMusic:

  """

  The DynamicMusic object allows to play music depending on a current scene, scene changes are managed by the class, with the possibility of custom transitions.
The default transition is a crossfade effect, and the default scene is silent and has id 0

  addScene(scene)

  Adds a new scene.

  addTransition(ini, end, transition)

  Adds a new scene.

  :arg ini:         
    the initial scene foor the transition.

  :type ini:        
    int

  :arg end:         
    The final scene for the transition.

  :type end:        
    int

  :arg transition:  
    The transition sound.

  :type transition: 
    :class:`Sound`

  :return:          
    false if the ini or end scenes don't exist, true othrwise.

  :rtype:           
    bool

  pause()

  Pauses playback of the scene.

  :return:          
    Whether the action succeeded.

  :rtype:           
    bool

  resume()

  Resumes playback of the scene.

  :return:          
    Whether the action succeeded.

  :rtype:           
    bool

  stop()

  Stops playback of the scene.

  :return:          
    Whether the action succeeded.

  :rtype:           
    bool

  """

  fadeTime: typing.Any = ...

  """

  The length in seconds of the crossfade transition

  """

  position: typing.Any = ...

  """

  The playback position of the scene in seconds.

  """

  scene: typing.Any = ...

  """

  The current scene

  """

  status: typing.Any = ...

  """

  Whether the scene is playing, paused or stopped (=invalid).

  """

  volume: typing.Any = ...

  """

  The volume of the scene.

  """

class Handle:

  """

  Handle objects are playback handles that can be used to control playback of a sound. If a sound is played back multiple times then there are as many handles.

  pause()

  Pauses playback.

  resume()

  Resumes playback.

  :return:          
    Whether the action succeeded.

  :rtype:           
    bool

  stop()

  Stops playback.

  :return:          
    Whether the action succeeded.

  :rtype:           
    bool

  Note: This makes the handle invalid.

  """

  attenuation: typing.Any = ...

  """

  This factor is used for distance based attenuation of the source.

  :attr:`Device.distance_model`

  """

  cone_angle_inner: typing.Any = ...

  """

  The opening angle of the inner cone of the source. If the cone values of a source are set there are two (audible) cones with the apex at the :attr:`location` of the source and with infinite height, heading in the direction of the source's :attr:`orientation`.
In the inner cone the volume is normal. Outside the outer cone the volume will be :attr:`cone_volume_outer` and in the area between the volume will be interpolated linearly.

  """

  cone_angle_outer: typing.Any = ...

  """

  The opening angle of the outer cone of the source.

  :attr:`cone_angle_inner`

  """

  cone_volume_outer: typing.Any = ...

  """

  The volume outside the outer cone of the source.

  :attr:`cone_angle_inner`

  """

  distance_maximum: typing.Any = ...

  """

  The maximum distance of the source.
If the listener is further away the source volume will be 0.

  :attr:`Device.distance_model`

  """

  distance_reference: typing.Any = ...

  """

  The reference distance of the source.
At this distance the volume will be exactly :attr:`volume`.

  :attr:`Device.distance_model`

  """

  keep: typing.Any = ...

  """

  Whether the sound should be kept paused in the device when its end is reached.
This can be used to seek the sound to some position and start playback again.

  Warning: If this is set to true and you forget stopping this equals a memory leak as the handle exists until the device is destroyed.

  """

  location: typing.Any = ...

  """

  The source's location in 3D space, a 3D tuple of floats.

  """

  loop_count: typing.Any = ...

  """

  The (remaining) loop count of the sound. A negative value indicates infinity.

  """

  orientation: typing.Any = ...

  """

  The source's orientation in 3D space as quaternion, a 4 float tuple.

  """

  pitch: typing.Any = ...

  """

  The pitch of the sound.

  """

  position: typing.Any = ...

  """

  The playback position of the sound in seconds.

  """

  relative: typing.Any = ...

  """

  Whether the source's location, velocity and orientation is relative or absolute to the listener.

  """

  status: typing.Any = ...

  """

  Whether the sound is playing, paused or stopped (=invalid).

  """

  velocity: typing.Any = ...

  """

  The source's velocity in 3D space, a 3D tuple of floats.

  """

  volume: typing.Any = ...

  """

  The volume of the sound.

  """

  volume_maximum: typing.Any = ...

  """

  The maximum volume of the source.

  :attr:`Device.distance_model`

  """

  volume_minimum: typing.Any = ...

  """

  The minimum volume of the source.

  :attr:`Device.distance_model`

  """

class PlaybackManager:

  """

  A PlabackManager object allows to easily control groups os sounds organized in categories.

  addCategory(volume)

  Adds a category with a custom volume.

  clean()

  Cleans all the invalid and finished sound from the playback manager.

  getVolume(catKey)

  Retrieves the volume of a category.

  :arg catKey:      
    the key of the category.

  :type catKey:     
    int

  :return:          
    The volume of the cateogry.

  :rtype:           
    float

  pause(catKey)

  Pauses playback of the category.

  :arg catKey:      
    the key of the category.

  :type catKey:     
    int

  :return:          
    Whether the action succeeded.

  :rtype:           
    bool

  setVolume(sound, catKey)

  Plays a sound through the playback manager and assigns it to a category.

  :arg sound:       
    The sound to play.

  :type sound:      
    :class:`Sound`

  :arg catKey:      
    the key of the category in which the sound will be added, if it doesn't exist, a new one will be created.

  :type catKey:     
    int

  :return:          
    The playback handle with which playback can be controlled with.

  :rtype:           
    :class:`Handle`

  resume(catKey)

  Resumes playback of the catgory.

  :arg catKey:      
    the key of the category.

  :type catKey:     
    int

  :return:          
    Whether the action succeeded.

  :rtype:           
    bool

  setVolume(volume, catKey)

  Changes the volume of a category.

  :arg volume:      
    the new volume value.

  :type volume:     
    float

  :arg catKey:      
    the key of the category.

  :type catKey:     
    int

  :return:          
    Whether the action succeeded.

  :rtype:           
    int

  stop(catKey)

  Stops playback of the category.

  :arg catKey:      
    the key of the category.

  :type catKey:     
    int

  :return:          
    Whether the action succeeded.

  :rtype:           
    bool

  """

  ...

class Sequence:

  """

  This sound represents sequenced entries to play a sound sequence.

  add()

  Adds a new entry to the sequence.

  remove()

  Removes an entry from the sequence.

  :arg entry:       
    The entry to remove.

  :type entry:      
    :class:`SequenceEntry`

  setAnimationData()

  Writes animation data to a sequence.

  :arg type:        
    The type of animation data.

  :type type:       
    int

  :arg frame:       
    The frame this data is for.

  :type frame:      
    int

  :arg data:        
    The data to write.

  :type data:       
    sequence of float

  :arg animated:    
    Whether the attribute is animated.

  :type animated:   
    bool

  """

  channels: typing.Any = ...

  """

  The channel count of the sequence.

  """

  distance_model: typing.Any = ...

  """

  The distance model of the sequence.

  http://connect.creativelabs.com/openal/Documentation/OpenAL%201.1%20Specification.htm#_Toc199835864

  """

  doppler_factor: typing.Any = ...

  """

  The doppler factor of the sequence.
This factor is a scaling factor for the velocity vectors in doppler calculation. So a value bigger than 1 will exaggerate the effect as it raises the velocity.

  """

  fps: typing.Any = ...

  """

  The listeners's location in 3D space, a 3D tuple of floats.

  """

  muted: typing.Any = ...

  """

  Whether the whole sequence is muted.

  """

  rate: typing.Any = ...

  """

  The sampling rate of the sequence in Hz.

  """

  speed_of_sound: typing.Any = ...

  """

  The speed of sound of the sequence.
The speed of sound in air is typically 343.3 m/s.

  """

class SequenceEntry:

  """

  SequenceEntry objects represent an entry of a sequenced sound.

  move()

  Moves the entry.

  setAnimationData()

  Writes animation data to a sequenced entry.

  :arg type:        
    The type of animation data.

  :type type:       
    int

  :arg frame:       
    The frame this data is for.

  :type frame:      
    int

  :arg data:        
    The data to write.

  :type data:       
    sequence of float

  :arg animated:    
    Whether the attribute is animated.

  :type animated:   
    bool

  """

  attenuation: typing.Any = ...

  """

  This factor is used for distance based attenuation of the source.

  :attr:`Device.distance_model`

  """

  cone_angle_inner: typing.Any = ...

  """

  The opening angle of the inner cone of the source. If the cone values of a source are set there are two (audible) cones with the apex at the :attr:`location` of the source and with infinite height, heading in the direction of the source's :attr:`orientation`.
In the inner cone the volume is normal. Outside the outer cone the volume will be :attr:`cone_volume_outer` and in the area between the volume will be interpolated linearly.

  """

  cone_angle_outer: typing.Any = ...

  """

  The opening angle of the outer cone of the source.

  :attr:`cone_angle_inner`

  """

  cone_volume_outer: typing.Any = ...

  """

  The volume outside the outer cone of the source.

  :attr:`cone_angle_inner`

  """

  distance_maximum: typing.Any = ...

  """

  The maximum distance of the source.
If the listener is further away the source volume will be 0.

  :attr:`Device.distance_model`

  """

  distance_reference: typing.Any = ...

  """

  The reference distance of the source.
At this distance the volume will be exactly :attr:`volume`.

  :attr:`Device.distance_model`

  """

  muted: typing.Any = ...

  """

  Whether the entry is muted.

  """

  relative: typing.Any = ...

  """

  Whether the source's location, velocity and orientation is relative or absolute to the listener.

  """

  sound: typing.Any = ...

  """

  The sound the entry is representing and will be played in the sequence.

  """

  volume_maximum: typing.Any = ...

  """

  The maximum volume of the source.

  :attr:`Device.distance_model`

  """

  volume_minimum: typing.Any = ...

  """

  The minimum volume of the source.

  :attr:`Device.distance_model`

  """

class Sound:

  """

  Sound objects are immutable and represent a sound that can be played simultaneously multiple times. They are called factories because they create reader objects internally that are used for playback.

  buffer(data, rate)

  Creates a sound from a data buffer.

  file(filename)

  Creates a sound object of a sound file.

  :arg filename:    
    Path of the file.

  :type filename:   
    string

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  Warning: If the file doesn't exist or can't be read you will not get an exception immediately, but when you try to start playback of that sound.

  list()

  Creates an empty sound list that can contain several sounds.

  :arg random:      
    wether the playback will be random or not.

  :type random:     
    int

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  sawtooth(frequency, rate=48000)

  Creates a sawtooth sound which plays a sawtooth wave.

  :arg frequency:   
    The frequency of the sawtooth wave in Hz.

  :type frequency:  
    float

  :arg rate:        
    The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping.

  :type rate:       
    int

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  silence(rate=48000)

  Creates a silence sound which plays simple silence.

  :arg rate:        
    The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping.

  :type rate:       
    int

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  sine(frequency, rate=48000)

  Creates a sine sound which plays a sine wave.

  :arg frequency:   
    The frequency of the sine wave in Hz.

  :type frequency:  
    float

  :arg rate:        
    The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping.

  :type rate:       
    int

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  square(frequency, rate=48000)

  Creates a square sound which plays a square wave.

  :arg frequency:   
    The frequency of the square wave in Hz.

  :type frequency:  
    float

  :arg rate:        
    The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping.

  :type rate:       
    int

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  triangle(frequency, rate=48000)

  Creates a triangle sound which plays a triangle wave.

  :arg frequency:   
    The frequency of the triangle wave in Hz.

  :type frequency:  
    float

  :arg rate:        
    The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping.

  :type rate:       
    int

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  ADSR(attack,decay,sustain,release)

  Attack-Decay-Sustain-Release envelopes the volume of a sound. Note: there is currently no way to trigger the release with this API.

  :arg attack:      
    The attack time in seconds.

  :type attack:     
    float

  :arg decay:       
    The decay time in seconds.

  :type decay:      
    float

  :arg sustain:     
    The sustain level.

  :type sustain:    
    float

  :arg release:     
    The release level.

  :type release:    
    float

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  accumulate(additive=False)

  Accumulates a sound by summing over positive input differences thus generating a monotonic sigal. If additivity is set to true negative input differences get added too, but positive ones with a factor of two. Note that with additivity the signal is not monotonic anymore.

  :arg additive:    
    Whether the accumulation should be additive or not.

  :type time:       
    bool

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  addSound(sound)

  Adds a new sound to a sound list.

  :arg sound:       
    The sound that will be added to the list.

  :type sound:      
    :class:`Sound`

  Note: You can only add a sound to a sound list.

  cache()

  Caches a sound into RAM.
This saves CPU usage needed for decoding and file access if the underlying sound reads from a file on the harddisk, but it consumes a lot of memory.

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  Note: Only known-length factories can be buffered.

  Warning: Raw PCM data needs a lot of space, only buffer short factories.

  data()

  Retrieves the data of the sound as numpy array.

  :return:          
    A two dimensional numpy float array.

  :rtype:           
    :class:`numpy.ndarray`

  Note: Best efficiency with cached sounds.

  delay(time)

  Delays by playing adding silence in front of the other sound's data.

  :arg time:        
    How many seconds of silence should be added before the sound.

  :type time:       
    float

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  envelope(attack, release, threshold, arthreshold)

  Delays by playing adding silence in front of the other sound's data.

  :arg attack:      
    The attack factor.

  :type attack:     
    float

  :arg release:     
    The release factor.

  :type release:    
    float

  :arg threshold:   
    The general threshold value.

  :type threshold:  
    float

  :arg arthreshold: 
    The attack/release threshold value.

  :type arthreshold:
    float

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  fadein(start, length)

  Fades a sound in by raising the volume linearly in the given time interval.

  :arg start:       
    Time in seconds when the fading should start.

  :type start:      
    float

  :arg length:      
    Time in seconds how long the fading should last.

  :type length:     
    float

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  Note: Before the fade starts it plays silence.

  fadeout(start, length)

  Fades a sound in by lowering the volume linearly in the given time interval.

  :arg start:       
    Time in seconds when the fading should start.

  :type start:      
    float

  :arg length:      
    Time in seconds how long the fading should last.

  :type length:     
    float

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  Note: After the fade this sound plays silence, so that the length of the sound is not altered.

  filter(b, a = (1))

  Filters a sound with the supplied IIR filter coefficients.
Without the second parameter you'll get a FIR filter.
If the first value of the a sequence is 0 it will be set to 1 automatically.
If the first value of the a sequence is neither 0 nor 1, all filter coefficients will be scaled by this value so that it is 1 in the end, you don't have to scale yourself.

  :arg b:           
    The nominator filter coefficients.

  :type b:          
    sequence of float

  :arg a:           
    The denominator filter coefficients.

  :type a:          
    sequence of float

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  highpass(frequency, Q=0.5)

  Creates a second order highpass filter based on the transfer function H(s) = s^2 / (s^2 + s/Q + 1)

  :arg frequency:   
    The cut off trequency of the highpass.

  :type frequency:  
    float

  :arg Q:           
    Q factor of the lowpass.

  :type Q:          
    float

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  join(sound)

  Plays two factories in sequence.

  :arg sound:       
    The sound to play second.

  :type sound:      
    :class:`Sound`

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  Note: The two factories have to have the same specifications (channels and samplerate).

  limit(start, end)

  Limits a sound within a specific start and end time.

  :arg start:       
    Start time in seconds.

  :type start:      
    float

  :arg end:         
    End time in seconds.

  :type end:        
    float

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  loop(count)

  Loops a sound.

  :arg count:       
    How often the sound should be looped. Negative values mean endlessly.

  :type count:      
    integer

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  Note: This is a filter function, you might consider using :attr:`Handle.loop_count` instead.

  lowpass(frequency, Q=0.5)

  Creates a second order lowpass filter based on the transfer function H(s) = 1 / (s^2 + s/Q + 1)

  :arg frequency:   
    The cut off trequency of the lowpass.

  :type frequency:  
    float

  :arg Q:           
    Q factor of the lowpass.

  :type Q:          
    float

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  mix(sound)

  Mixes two factories.

  :arg sound:       
    The sound to mix over the other.

  :type sound:      
    :class:`Sound`

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  Note: The two factories have to have the same specifications (channels and samplerate).

  modulate(sound)

  Modulates two factories.

  :arg sound:       
    The sound to modulate over the other.

  :type sound:      
    :class:`Sound`

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  Note: The two factories have to have the same specifications (channels and samplerate).

  mutable()

  Creates a sound that will be restarted when sought backwards.
If the original sound is a sound list, the playing sound can change.

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  pingpong()

  Plays a sound forward and then backward.
This is like joining a sound with its reverse.

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  pitch(factor)

  Changes the pitch of a sound with a specific factor.

  :arg factor:      
    The factor to change the pitch with.

  :type factor:     
    float

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  Note: This is done by changing the sample rate of the underlying sound, which has to be an integer, so the factor value rounded and the factor may not be 100 % accurate.

  Note: This is a filter function, you might consider using :attr:`Handle.pitch` instead.

  rechannel(channels)

  Rechannels the sound.

  :arg channels:    
    The new channel configuration.

  :type channels:   
    int

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  resample(rate, high_quality)

  Resamples the sound.

  :arg rate:        
    The new sample rate.

  :type rate:       
    double

  :arg high_quality:
    When true use a higher quality but slower resampler.

  :type high_quality:
    bool

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  reverse()

  Plays a sound reversed.

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  Note: The sound has to have a finite length and has to be seekable. It's recommended to use this only with factories      with fast and accurate seeking, which is not true for encoded audio files, such ones should be buffered using :meth:`cache` before being played reversed.

  Warning: If seeking is not accurate in the underlying sound you'll likely hear skips/jumps/cracks.

  sum()

  Sums the samples of a sound.

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  threshold(threshold = 0)

  Makes a threshold wave out of an audio wave by setting all samples with a amplitude >= threshold to 1, all <= -threshold to -1 and all between to 0.

  :arg threshold:   
    Threshold value over which an amplitude counts non-zero.

  :type threshold:  
    float

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  volume(volume)

  Changes the volume of a sound.

  :arg volume:      
    The new volume..

  :type volume:     
    float

  :return:          
    The created :class:`Sound` object.

  :rtype:           
    :class:`Sound`

  Note: Should be in the range [0, 1] to avoid clipping.

  Note: This is a filter function, you might consider using :attr:`Handle.volume` instead.

  write(filename, rate, channels, format, container, codec, bitrate, buffersize)

  Writes the sound to a file.

  :arg filename:    
    The path to write to.

  :type filename:   
    string

  :arg rate:        
    The sample rate to write with.

  :type rate:       
    int

  :arg channels:    
    The number of channels to write with.

  :type channels:   
    int

  :arg format:      
    The sample format to write with.

  :type format:     
    int

  :arg container:   
    The container format for the file.

  :type container:  
    int

  :arg codec:       
    The codec to use in the file.

  :type codec:      
    int

  :arg bitrate:     
    The bitrate to write with.

  :type bitrate:    
    int

  :arg buffersize:  
    The size of the writing buffer.

  :type buffersize: 
    int

  """

  length: typing.Any = ...

  """

  The sample specification of the sound as a tuple with rate and channel count.

  """

  specs: typing.Any = ...

  """

  The sample specification of the sound as a tuple with rate and channel count.

  """

class Source:

  """

  The source object represents the source position of a binaural sound.

  """

  azimuth: typing.Any = ...

  """

  The azimuth angle.

  """

  distance: typing.Any = ...

  """

  The distance value. 0 is min, 1 is max.

  """

  elevation: typing.Any = ...

  """

  The elevation angle.

  """

class ThreadPool:

  """

  A ThreadPool is used to parallelize convolution efficiently.

  """

  ...

class error:

  ...



"""


Sculpt Operators
****************

:func:`brush_stroke`

:func:`detail_flood_fill`

:func:`dirty_mask`

:func:`dynamic_topology_toggle`

:func:`mask_expand`

:func:`mask_filter`

:func:`mesh_filter`

:func:`optimize`

:func:`sample_detail_size`

:func:`sculptmode_toggle`

:func:`set_detail_size`

:func:`set_persistent_base`

:func:`set_pivot_position`

:func:`symmetrize`

:func:`uv_sculpt_stroke`

"""

import typing

def brush_stroke(stroke: typing.Union[typing.Sequence[OperatorStrokeElement], typing.Mapping[str, OperatorStrokeElement], bpy.types.bpy_prop_collection] = None, mode: str = 'NORMAL', ignore_background_click: bool = False) -> None:

  """

  Sculpt a stroke into the geometry

  """

  ...

def detail_flood_fill() -> None:

  """

  Flood fill the mesh with the selected detail setting

  """

  ...

def dirty_mask(dirty_only: bool = False) -> None:

  """

  Generates a mask based on the geometry cavity and pointiness

  """

  ...

def dynamic_topology_toggle() -> None:

  """

  Dynamic topology alters the mesh topology while sculpting

  """

  ...

def mask_expand(invert: bool = True, use_cursor: bool = True, update_pivot: bool = True, smooth_iterations: int = 2, mask_speed: int = 5, use_normals: bool = True, keep_previous_mask: bool = False, edge_sensitivity: int = 300) -> None:

  """

  Expands a mask from the initial active vertex under the cursor

  """

  ...

def mask_filter(filter_type: str = 'SMOOTH', iterations: int = 1, auto_iteration_count: bool = False) -> None:

  """

  Applies a filter to modify the current mask

  """

  ...

def mesh_filter(type: str = 'INFLATE', strength: float = 1.0, deform_axis: typing.Set[str] = {'X', 'Y', 'Z'}) -> None:

  """

  Applies a filter to modify the current mesh

  """

  ...

def optimize() -> None:

  """

  Recalculate the sculpt BVH to improve performance

  """

  ...

def sample_detail_size(location: typing.Tuple[int, int] = (0, 0)) -> None:

  """

  Sample the mesh detail on clicked point

  """

  ...

def sculptmode_toggle() -> None:

  """

  Toggle sculpt mode in 3D view

  """

  ...

def set_detail_size() -> None:

  """

  Set the mesh detail (either relative or constant one, depending on current dyntopo mode)

  """

  ...

def set_persistent_base() -> None:

  """

  Reset the copy of the mesh that is being sculpted on

  """

  ...

def set_pivot_position(mode: str = 'UNMASKED') -> None:

  """

  Sets the sculpt transform pivot position

  """

  ...

def symmetrize() -> None:

  """

  Symmetrize the topology modifications

  """

  ...

def uv_sculpt_stroke(mode: str = 'NORMAL') -> None:

  """

  Sculpt UVs using a brush

  """

  ...



"""


Gpencil Operators
*****************

:func:`active_frame_delete`

:func:`active_frames_delete_all`

:func:`annotate`

:func:`blank_frame_add`

:func:`brush_presets_create`

:func:`color_hide`

:func:`color_isolate`

:func:`color_lock_all`

:func:`color_reveal`

:func:`color_select`

:func:`color_unlock_all`

:func:`convert`

:func:`convert_old_files`

:func:`copy`

:func:`data_add`

:func:`data_unlink`

:func:`delete`

:func:`dissolve`

:func:`draw`

:func:`duplicate`

:func:`duplicate_move`

:func:`editmode_toggle`

:func:`extrude`

:func:`extrude_move`

:func:`fill`

:func:`frame_clean_fill`

:func:`frame_clean_loose`

:func:`frame_duplicate`

:func:`generate_weights`

:func:`guide_rotate`

:func:`hide`

:func:`interpolate`

:func:`interpolate_reverse`

:func:`interpolate_sequence`

:func:`layer_add`

:func:`layer_change`

:func:`layer_duplicate`

:func:`layer_duplicate_object`

:func:`layer_isolate`

:func:`layer_merge`

:func:`layer_move`

:func:`layer_remove`

:func:`lock_all`

:func:`lock_layer`

:func:`move_to_layer`

:func:`paintmode_toggle`

:func:`paste`

:func:`primitive`

:func:`reproject`

:func:`reveal`

:func:`sculpt_paint`

:func:`sculptmode_toggle`

:func:`select`

:func:`select_all`

:func:`select_alternate`

:func:`select_box`

:func:`select_circle`

:func:`select_first`

:func:`select_grouped`

:func:`select_lasso`

:func:`select_last`

:func:`select_less`

:func:`select_linked`

:func:`select_more`

:func:`selection_opacity_toggle`

:func:`selectmode_toggle`

:func:`set_active_material`

:func:`snap_cursor_to_selected`

:func:`snap_to_cursor`

:func:`snap_to_grid`

:func:`stroke_apply_thickness`

:func:`stroke_arrange`

:func:`stroke_caps_set`

:func:`stroke_change_color`

:func:`stroke_cutter`

:func:`stroke_cyclical_set`

:func:`stroke_flip`

:func:`stroke_join`

:func:`stroke_lock_color`

:func:`stroke_merge`

:func:`stroke_merge_by_distance`

:func:`stroke_sample`

:func:`stroke_separate`

:func:`stroke_simplify`

:func:`stroke_simplify_fixed`

:func:`stroke_smooth`

:func:`stroke_split`

:func:`stroke_subdivide`

:func:`stroke_trim`

:func:`unlock_all`

:func:`vertex_group_assign`

:func:`vertex_group_deselect`

:func:`vertex_group_invert`

:func:`vertex_group_normalize`

:func:`vertex_group_normalize_all`

:func:`vertex_group_remove_from`

:func:`vertex_group_select`

:func:`vertex_group_smooth`

:func:`weightmode_toggle`

"""

import typing

def active_frame_delete() -> None:

  """

  Delete the active frame for the active Grease Pencil Layer

  """

  ...

def active_frames_delete_all() -> None:

  """

  Delete the active frame(s) of all editable Grease Pencil layers

  """

  ...

def annotate(mode: str = 'DRAW', stroke: typing.Union[typing.Sequence[OperatorStrokeElement], typing.Mapping[str, OperatorStrokeElement], bpy.types.bpy_prop_collection] = None, wait_for_input: bool = True) -> None:

  """

  Make annotations on the active data

  """

  ...

def blank_frame_add(all_layers: bool = False) -> None:

  """

  Insert a blank frame on the current frame (all subsequently existing frames, if any, are shifted right by one frame)

  """

  ...

def brush_presets_create() -> None:

  """

  Create a set of predefined Grease Pencil drawing brushes

  """

  ...

def color_hide(unselected: bool = False) -> None:

  """

  Hide selected/unselected Grease Pencil colors

  """

  ...

def color_isolate(affect_visibility: bool = False) -> None:

  """

  Toggle whether the active color is the only one that is editable and/or visible

  """

  ...

def color_lock_all() -> None:

  """

  Lock all Grease Pencil colors to prevent them from being accidentally modified

  """

  ...

def color_reveal() -> None:

  """

  Unhide all hidden Grease Pencil colors

  """

  ...

def color_select(deselect: bool = False) -> None:

  """

  Select all Grease Pencil strokes using current color

  """

  ...

def color_unlock_all() -> None:

  """

  Unlock all Grease Pencil colors so that they can be edited

  """

  ...

def convert(type: str = 'PATH', use_normalize_weights: bool = True, radius_multiplier: float = 1.0, use_link_strokes: bool = False, timing_mode: str = 'FULL', frame_range: int = 100, start_frame: int = 1, use_realtime: bool = False, end_frame: int = 250, gap_duration: float = 0.0, gap_randomness: float = 0.0, seed: int = 0, use_timing_data: bool = False) -> None:

  """

  Convert the active Grease Pencil layer to a new Curve Object

  """

  ...

def convert_old_files(annotation: bool = False) -> None:

  """

  Convert 2.7x grease pencil files to 2.80

  """

  ...

def copy() -> None:

  """

  Copy selected Grease Pencil points and strokes

  """

  ...

def data_add() -> None:

  """

  Add new Grease Pencil data-block

  """

  ...

def data_unlink() -> None:

  """

  Unlink active Annotation data-block

  """

  ...

def delete(type: str = 'POINTS') -> None:

  """

  Delete selected Grease Pencil strokes, vertices, or frames

  """

  ...

def dissolve(type: str = 'POINTS') -> None:

  """

  Delete selected points without splitting strokes

  """

  ...

def draw(mode: str = 'DRAW', stroke: typing.Union[typing.Sequence[OperatorStrokeElement], typing.Mapping[str, OperatorStrokeElement], bpy.types.bpy_prop_collection] = None, wait_for_input: bool = True, disable_straight: bool = False, disable_fill: bool = False, guide_last_angle: float = 0.0) -> None:

  """

  Draw a new stroke in the active Grease Pencil Object

  """

  ...

def duplicate() -> None:

  """

  Duplicate the selected Grease Pencil strokes

  """

  ...

def duplicate_move(GPENCIL_OT_duplicate: GPENCIL_OT_duplicate = None, TRANSFORM_OT_translate: TRANSFORM_OT_translate = None) -> None:

  """

  Make copies of the selected Grease Pencil strokes and move them

  """

  ...

def editmode_toggle(back: bool = False) -> None:

  """

  Enter/Exit edit mode for Grease Pencil strokes

  """

  ...

def extrude() -> None:

  """

  Extrude the selected Grease Pencil points

  """

  ...

def extrude_move(GPENCIL_OT_extrude: GPENCIL_OT_extrude = None, TRANSFORM_OT_translate: TRANSFORM_OT_translate = None) -> None:

  """

  Extrude selected points and move them

  """

  ...

def fill(on_back: bool = False) -> None:

  """

  Fill with color the shape formed by strokes

  """

  ...

def frame_clean_fill(mode: str = 'ACTIVE') -> None:

  """

  Remove 'no fill' boundary strokes

  """

  ...

def frame_clean_loose(limit: int = 1) -> None:

  """

  Remove loose points

  """

  ...

def frame_duplicate(mode: str = 'ACTIVE') -> None:

  """

  Make a copy of the active Grease Pencil Frame

  """

  ...

def generate_weights(mode: str = 'NAME', armature: str = 'DEFAULT', ratio: float = 0.1, decay: float = 0.8) -> None:

  """

  Generate automatic weights for armatures (requires armature modifier)

  """

  ...

def guide_rotate(increment: bool = True, angle: float = 0.0) -> None:

  """

  Rotate guide angle

  """

  ...

def hide(unselected: bool = False) -> None:

  """

  Hide selected/unselected Grease Pencil layers

  """

  ...

def interpolate(shift: float = 0.0) -> None:

  """

  Interpolate grease pencil strokes between frames

  """

  ...

def interpolate_reverse() -> None:

  """

  Remove breakdown frames generated by interpolating between two Grease Pencil frames

  """

  ...

def interpolate_sequence() -> None:

  """

  Generate 'in-betweens' to smoothly interpolate between Grease Pencil frames

  """

  ...

def layer_add() -> None:

  """

  Add new layer or note for the active data-block

  """

  ...

def layer_change(layer: str = 'DEFAULT') -> None:

  """

  Change active Grease Pencil layer

  """

  ...

def layer_duplicate() -> None:

  """

  Make a copy of the active Grease Pencil layer

  """

  ...

def layer_duplicate_object(object: str = '', mode: str = 'ALL') -> None:

  """

  Make a copy of the active Grease Pencil layer to new object

  """

  ...

def layer_isolate(affect_visibility: bool = False) -> None:

  """

  Toggle whether the active layer is the only one that can be edited and/or visible

  """

  ...

def layer_merge() -> None:

  """

  Merge the current layer with the layer below

  """

  ...

def layer_move(type: str = 'UP') -> None:

  """

  Move the active Grease Pencil layer up/down in the list

  """

  ...

def layer_remove() -> None:

  """

  Remove active Grease Pencil layer

  """

  ...

def lock_all() -> None:

  """

  Lock all Grease Pencil layers to prevent them from being accidentally modified

  """

  ...

def lock_layer() -> None:

  """

  Lock and hide any color not used in any layer

  """

  ...

def move_to_layer(layer: int = 0) -> None:

  """

  Move selected strokes to another layer

  """

  ...

def paintmode_toggle(back: bool = False) -> None:

  """

  Enter/Exit paint mode for Grease Pencil strokes

  """

  ...

def paste(type: str = 'ACTIVE') -> None:

  """

  Paste previously copied strokes to active layer or to original layer

  """

  ...

def primitive(edges: int = 4, type: str = 'BOX', wait_for_input: bool = True) -> None:

  """

  Create predefined grease pencil stroke shapes

  """

  ...

def reproject(type: str = 'VIEW') -> None:

  """

  Reproject the selected strokes from the current viewpoint as if they had been newly drawn (e.g. to fix problems from accidental 3D cursor movement or accidental viewport changes, or for matching deforming geometry)

  """

  ...

def reveal(select: bool = True) -> None:

  """

  Show all Grease Pencil layers

  """

  ...

def sculpt_paint(stroke: typing.Union[typing.Sequence[OperatorStrokeElement], typing.Mapping[str, OperatorStrokeElement], bpy.types.bpy_prop_collection] = None, wait_for_input: bool = True) -> None:

  """

  Apply tweaks to strokes by painting over the strokes

  """

  ...

def sculptmode_toggle(back: bool = False) -> None:

  """

  Enter/Exit sculpt mode for Grease Pencil strokes

  """

  ...

def select(extend: bool = False, deselect: bool = False, toggle: bool = False, deselect_all: bool = False, entire_strokes: bool = False, location: typing.Tuple[int, int] = (0, 0)) -> None:

  """

  Select Grease Pencil strokes and/or stroke points

  """

  ...

def select_all(action: str = 'TOGGLE') -> None:

  """

  Change selection of all Grease Pencil strokes currently visible

  """

  ...

def select_alternate(unselect_ends: bool = True) -> None:

  """

  Select alternative points in same strokes as already selected points

  """

  ...

def select_box(xmin: int = 0, xmax: int = 0, ymin: int = 0, ymax: int = 0, wait_for_input: bool = True, mode: str = 'SET') -> None:

  """

  Select Grease Pencil strokes within a rectangular region

  """

  ...

def select_circle(x: int = 0, y: int = 0, radius: int = 25, wait_for_input: bool = True, mode: str = 'SET') -> None:

  """

  Select Grease Pencil strokes using brush selection

  """

  ...

def select_first(only_selected_strokes: bool = False, extend: bool = False) -> None:

  """

  Select first point in Grease Pencil strokes

  """

  ...

def select_grouped(type: str = 'LAYER') -> None:

  """

  Select all strokes with similar characteristics

  """

  ...

def select_lasso(mode: str = 'SET', path: typing.Union[typing.Sequence[OperatorMousePath], typing.Mapping[str, OperatorMousePath], bpy.types.bpy_prop_collection] = None) -> None:

  """

  Select Grease Pencil strokes using lasso selection

  """

  ...

def select_last(only_selected_strokes: bool = False, extend: bool = False) -> None:

  """

  Select last point in Grease Pencil strokes

  """

  ...

def select_less() -> None:

  """

  Shrink sets of selected Grease Pencil points

  """

  ...

def select_linked() -> None:

  """

  Select all points in same strokes as already selected points

  """

  ...

def select_more() -> None:

  """

  Grow sets of selected Grease Pencil points

  """

  ...

def selection_opacity_toggle() -> None:

  """

  Hide/Unhide selected points for Grease Pencil strokes setting alpha factor

  """

  ...

def selectmode_toggle(mode: int = 0) -> None:

  """

  Set selection mode for Grease Pencil strokes

  """

  ...

def set_active_material() -> None:

  """

  Set the selected stroke material as the active material

  """

  ...

def snap_cursor_to_selected() -> None:

  """

  Snap cursor to center of selected points

  """

  ...

def snap_to_cursor(use_offset: bool = True) -> None:

  """

  Snap selected points/strokes to the cursor

  """

  ...

def snap_to_grid() -> None:

  """

  Snap selected points to the nearest grid points

  """

  ...

def stroke_apply_thickness() -> None:

  """

  Apply the thickness change of the layer to its strokes

  """

  ...

def stroke_arrange(direction: str = 'UP') -> None:

  """

  Arrange selected strokes up/down in the drawing order of the active layer

  """

  ...

def stroke_caps_set(type: str = 'TOGGLE') -> None:

  """

  Change Stroke caps mode (rounded or flat)

  """

  ...

def stroke_change_color(material: str = '') -> None:

  """

  Move selected strokes to active material

  """

  ...

def stroke_cutter(path: typing.Union[typing.Sequence[OperatorMousePath], typing.Mapping[str, OperatorMousePath], bpy.types.bpy_prop_collection] = None) -> None:

  """

  Select section and cut

  """

  ...

def stroke_cyclical_set(type: str = 'TOGGLE', geometry: bool = False) -> None:

  """

  Close or open the selected stroke adding an edge from last to first point

  """

  ...

def stroke_flip() -> None:

  """

  Change direction of the points of the selected strokes

  """

  ...

def stroke_join(type: str = 'JOIN', leave_gaps: bool = False) -> None:

  """

  Join selected strokes (optionally as new stroke)

  """

  ...

def stroke_lock_color() -> None:

  """

  Lock any color not used in any selected stroke

  """

  ...

def stroke_merge(mode: str = 'STROKE', back: bool = False, additive: bool = False, cyclic: bool = False, clear_point: bool = False, clear_stroke: bool = False) -> None:

  """

  Create a new stroke with the selected stroke points

  """

  ...

def stroke_merge_by_distance(threshold: float = 0.001, use_unselected: bool = False) -> None:

  """

  Merge points by distance

  """

  ...

def stroke_sample(length: float = 0.1) -> None:

  """

  Sample stroke points to predefined segment length

  """

  ...

def stroke_separate(mode: str = 'POINT') -> None:

  """

  Separate the selected strokes or layer in a new grease pencil object

  """

  ...

def stroke_simplify(factor: float = 0.0) -> None:

  """

  Simplify selected stroked reducing number of points

  """

  ...

def stroke_simplify_fixed(step: int = 1) -> None:

  """

  Simplify selected stroked reducing number of points using fixed algorithm

  """

  ...

def stroke_smooth(repeat: int = 1, factor: float = 0.5, only_selected: bool = True, smooth_position: bool = True, smooth_thickness: bool = True, smooth_strength: bool = False, smooth_uv: bool = False) -> None:

  """

  Smooth selected strokes

  """

  ...

def stroke_split() -> None:

  """

  Split selected points as new stroke on same frame

  """

  ...

def stroke_subdivide(number_cuts: int = 1, factor: float = 0.0, repeat: int = 1, only_selected: bool = True, smooth_position: bool = True, smooth_thickness: bool = True, smooth_strength: bool = False, smooth_uv: bool = False) -> None:

  """

  Subdivide between continuous selected points of the stroke adding a point half way between them

  """

  ...

def stroke_trim() -> None:

  """

  Trim selected stroke to first loop or intersection

  """

  ...

def unlock_all() -> None:

  """

  Unlock all Grease Pencil layers so that they can be edited

  """

  ...

def vertex_group_assign() -> None:

  """

  Assign the selected vertices to the active vertex group

  """

  ...

def vertex_group_deselect() -> None:

  """

  Deselect all selected vertices assigned to the active vertex group

  """

  ...

def vertex_group_invert() -> None:

  """

  Invert weights to the active vertex group

  """

  ...

def vertex_group_normalize() -> None:

  """

  Normalize weights to the active vertex group

  """

  ...

def vertex_group_normalize_all(lock_active: bool = True) -> None:

  """

  Normalize all weights of all vertex groups, so that for each vertex, the sum of all weights is 1.0

  """

  ...

def vertex_group_remove_from() -> None:

  """

  Remove the selected vertices from active or all vertex group(s)

  """

  ...

def vertex_group_select() -> None:

  """

  Select all the vertices assigned to the active vertex group

  """

  ...

def vertex_group_smooth(factor: float = 0.5, repeat: int = 1) -> None:

  """

  Smooth weights to the active vertex group

  """

  ...

def weightmode_toggle(back: bool = False) -> None:

  """

  Enter/Exit weight paint mode for Grease Pencil strokes

  """

  ...



"""


BMesh Module (bmesh)
********************

This module provides access to blenders bmesh data structures.

Submodules:


Introduction
============

This API gives access the blenders internal mesh editing api, featuring geometry connectivity data and
access to editing operations such as split, separate, collapse and dissolve.

The features exposed closely follow the C API,
giving python access to the functions used by blenders own mesh editing tools.

For an overview of BMesh data types and how they reference each other see:
`BMesh Design Document <https://wiki.blender.org/index.php/Dev:Source/Modeling/BMesh/Design>`_ .

Note: **Disk** and **Radial** data is not exposed by the python api since this is for internal use only.

Warning: TODO items are...

  * add access to BMesh **walkers**

  * add custom-data manipulation functions add/remove/rename.


Example Script
==============

.. code::

  # This example assumes we have a mesh object selected

  import bpy
  import bmesh

  # Get the active mesh
  me = bpy.context.object.data


  # Get a BMesh representation
  bm = bmesh.new()   # create an empty BMesh
  bm.from_mesh(me)   # fill it in from a Mesh


  # Modify the BMesh, can do anything here...
  for v in bm.verts:
      v.co.x += 1.0


  # Finish up, write the bmesh back to the mesh
  bm.to_mesh(me)
  bm.free()  # free and prevent further access


Stand-Alone Module
------------------

The bmesh module is written to be standalone except for :mod:`mathutils`
which is used for vertex locations and normals.

The only other exception to this are when converting mesh data to and from :class:`bpy.types.Mesh`.


Mesh Access
===========

There are 2 ways to access BMesh data, you can create a new BMesh by converting a mesh from
:class:`bpy.types.BlendData.meshes` or by accessing the current edit mode mesh.
see: :class:`bmesh.types.BMesh.from_mesh` and :mod:`bmesh.from_edit_mesh` respectively.

When explicitly converting from mesh data python **owns** the data, that is to say -
that the mesh only exists while python holds a reference to it,
and the script is responsible for putting it back into a mesh data-block when the edits are done.

Note that unlike :mod:`bpy`, a BMesh does not necessarily correspond to data in the currently open blend file,
a BMesh can be created, edited and freed without the user ever seeing or having access to it.
Unlike edit mode, the bmesh module can use multiple BMesh instances at once.

Take care when dealing with multiple BMesh instances since the mesh data can use a lot of memory, while a mesh that
python owns will be freed in when the script holds no references to it,
its good practice to call :class:`bmesh.types.BMesh.free` which will remove all the mesh data immediately and disable
further access.


EditMode Tessellation
---------------------

When writing scripts that operate on editmode data you will normally want to re-calculate the tessellation after
running the  script, this needs to be called explicitly.

The BMesh its self does not store the triangulated faces, they are stored in the :class:`bpy.types.Mesh`,
to refresh tessellation triangles call :class:`bpy.types.Mesh.calc_loop_triangles`.


CustomData Access
=================

BMesh has a unified way to access mesh attributes such as UV's vertex colors, shape keys, edge crease etc.

This works by having a **layers** property on bmesh data sequences to access the custom data layers which can then be
used to access the actual data on each vert/edge/face/loop.

Here are some examples ...

.. code:: python

  uv_lay = bm.loops.layers.uv.active

  for face in bm.faces:
      for loop in face.loops:
          uv = loop[uv_lay].uv
          print("Loop UV: %f, %f" % uv[:])
          vert = loop.vert
          print("Loop Vert: (%f,%f,%f)" % vert.co[:])

.. code:: python

  shape_lay = bm.verts.layers.shape["Key.001"]

  for vert in bm.verts:
      shape = vert[shape_lay]
      print("Vert Shape: %f, %f, %f" % (shape.x, shape.y, shape.z))

.. code:: python

  # in this example the active vertex group index is used,
  # this is stored in the object, not the BMesh
  group_index = obj.vertex_groups.active_index

  # only ever one deform weight layer
  dvert_lay = bm.verts.layers.deform.active

  for vert in bm.verts:
      dvert = vert[dvert_lay]

      if group_index in dvert:
          print("Weight %f" % dvert[group_index])
      else:
          print("Setting Weight")
          dvert[group_index] = 0.5


Keeping a Correct State
=======================

When modeling in blender there are certain assumptions made about the state of the mesh.

* hidden geometry isn't selected.

* when an edge is selected, its vertices are selected too.

* when a face is selected, its edges and vertices are selected.

* duplicate edges / faces don't exist.

* faces have at least 3 vertices.

To give developers flexibility these conventions are not enforced,
however tools must leave the mesh in a valid state else other tools may behave incorrectly.

Any errors that arise from not following these conventions is considered a bug in the script,
not a bug in blender.


Selection / Flushing
--------------------

As mentioned above, it is possible to create an invalid selection state
(by selecting a state and then de-selecting one of its vertices's for example), mostly the best way to solve this is to
flush the selection after performing a series of edits. this validates the selection state.


Module Functions
================

:func:`from_edit_mesh`

:func:`new`

:func:`update_edit_mesh`

"""

from . import utils

from . import types

from . import ops

from . import geometry

import typing

import bpy

def from_edit_mesh(self, mesh: bpy.types.Mesh) -> types.BMesh:

  """

  Return a BMesh from this mesh, currently the mesh must already be in editmode.

  """

  ...

def new(self, use_operators: bool = True) -> types.BMesh:

  ...

def update_edit_mesh(self, mesh: bpy.types.Mesh, loop_triangles: bool = True, destructive: bool = True) -> None:

  """

  Update the mesh after changes to the BMesh in editmode,
optionally recalculating n-gon tessellation.

  """

  ...



"""


Buttons Operators
*****************

:func:`context_menu`

:func:`directory_browse`

:func:`file_browse`

"""

import typing

def context_menu() -> None:

  """

  Display properties editor context_menu

  """

  ...

def directory_browse(directory: str = '', hide_props_region: bool = True, filter_blender: bool = False, filter_backup: bool = False, filter_image: bool = False, filter_movie: bool = False, filter_python: bool = False, filter_font: bool = False, filter_sound: bool = False, filter_text: bool = False, filter_archive: bool = False, filter_btx: bool = False, filter_collada: bool = False, filter_alembic: bool = False, filter_folder: bool = False, filter_blenlib: bool = False, filemode: int = 9, relative_path: bool = True, display_type: str = 'DEFAULT', sort_method: str = 'FILE_SORT_ALPHA') -> None:

  """

  Open a directory browser, Hold Shift to open the file, Alt to browse containing directory

  """

  ...

def file_browse(filepath: str = '', hide_props_region: bool = True, filter_blender: bool = False, filter_backup: bool = False, filter_image: bool = False, filter_movie: bool = False, filter_python: bool = False, filter_font: bool = False, filter_sound: bool = False, filter_text: bool = False, filter_archive: bool = False, filter_btx: bool = False, filter_collada: bool = False, filter_alembic: bool = False, filter_folder: bool = False, filter_blenlib: bool = False, filemode: int = 9, relative_path: bool = True, display_type: str = 'DEFAULT', sort_method: str = 'FILE_SORT_ALPHA') -> None:

  """

  Open a file browser, Hold Shift to open the file, Alt to browse containing directory

  """

  ...
